%%
%  Author: Peter Polcz (ppolcz@gmail.com) 
%  Created on 2023. September 21. (2023a)
%

% The ODE looks like:
% 
% dx/dt = Eq(t,x)
% 
mu = 1;
Eq = @(t,x) [
    x(2)
    mu*(1-x(1).^2).*x(2) - x(1)
    ];

x0 = [3;0];

T = 10;

% The ODE is solved HERE:
[t,x_sol] = ode89(Eq,[0,T],x0);

fig = figure(319);
fig.Position(3:4) = [1453 490];
delete(fig.Children)

Tl = tiledlayout(1,2,"TileSpacing","compact","Padding","tight");

ax1 = nexttile;
hold on; grid on; box on;

% x1 = x_sol(:,1) --> the first column of x_sol
% x2 = x_sol(:,2) --> the second column of x_sol

% b     blue          .     point              -     solid
% g     green         o     circle             :     dotted
% r     red           x     x-mark             -.    dashdot 
% c     cyan          +     plus               --    dashed   
% m     magenta       *     star             (none)  no line
% y     yellow        s     square
% k     black         d     diamond
% w     white         v     triangle (down)
%                     ^     triangle (up)
%                     <     triangle (left)
%                     >     triangle (right)
%                     p     pentagram
%                     h     hexagram
Pl = plot(x_sol(:,1),x_sol(:,2),'b.--');
Pl0 = plot(x_sol(1,1),x_sol(1,2),'.','MarkerSize',15,'Color',Pl.Color);

xlabel('x1')
ylabel('x2')

ax2 = nexttile;
hold on; grid on; box on;

% sin, cos, and sin+cos are added just for demonstration
plot(t,[ x_sol , sin(t) , cos(t) , sin(t)+cos(t) ])

xlabel('t')
ylabel('x1 and x2')

legend('x1','x2','sin(t)','cos(t)','sin(t)+cos(t)')



























%% Other attempts not related to Van der Pol


f = @(t,x) [
    1 - x(1)
    1
    ];

fig = figure(123);
delete(fig.Children)
ax = axes(fig);
hold on

[t,x] = ode45(f,[0,10],[0.5;0]);
x1 = x(:,1).*cos(x(:,2));
x2 = x(:,1).*sin(x(:,2));
plot(x1,x2)

[t,x] = ode45(f,[0,10],[1.5;0]);
x1 = x(:,1).*cos(x(:,2));
x2 = x(:,1).*sin(x(:,2));
plot(x1,x2)


