%%
%  Author: Peter Polcz (ppolcz@gmail.com) 
%  Created on 2023. October 26. (2023a)
%

clear x
syms t x y real

A = 1/8;
mu = 1/2;
f = [
    A - x + x^2*y
    mu - x^2*y
    ];

V = x + y;
x = [x;y];

YLim = 45;
XLim = 45;

f_fh = matlabFunction(f,'vars',{t,x});
J_fh = matlabFunction(jacobian(f,x),'vars',{x});
f1_fh = matlabFunction(f(1),'vars',x);
f2_fh = matlabFunction(f(2),'vars',x);
V_fh = matlabFunction(V);

dV = jacobian(V,x)*f;
dV_fh = matlabFunction(dV,'vars',x);

sol = solve(f,x);
Eq = double([sol.x,sol.y]');


figure(1);
Tl = tiledlayout(3,3,"TileSpacing","compact","Padding","compact");

%%% 
% 1. 

Ax1 = nexttile;
hold on; grid on; box on;

x = linspace(0.1,XLim,1000);
y = (x - A) ./ x.^2;
plot(x,y)

y = mu ./ x.^2;
plot(x,y)

plot(Eq(1),Eq(2),'.','MarkerSize',25)

title('Problem 1. Equilibrium point')


%%%
% 2.

Ax1 = nexttile([2 2]);
hold on; grid on; box on;

Res = 501;
[x,y] = meshgrid(linspace(0,XLim,Res),linspace(0,YLim,Res));
f1 = f1_fh(x,y);
f2 = f2_fh(x,y);
n = sqrt(f1.^2 + f2.^2);
f1 = f1 ./ n;
f2 = f2 ./ n;

quiver(x,y,f1,f2);
plot(Eq(1),Eq(2),'.','MarkerSize',25)

xlim([0,XLim])
ylim([0,YLim])

title('Problem 2. Vector field')


%%%
% 4.

a = A - 0.001;
m = mu/a^2 + 0.01;
c = m + 4;

LS = [
    a,a,c-m-a,c,a
    0,m,    m,0,0
    ]';


t = linspace(0,4,1000);
LS_ = interp1(0:4,LS,t);

plot(LS(:,1),LS(:,2),'r');

dV_val = dV_fh(LS_(:,1),LS(:,2));

%%%
% 5.

[~,x] = ode89(f_fh,[0,100],[5,25]);
Pl = plot(x(:,1),x(:,2));
plot(x(1,1),x(1,2),'.','Color',Pl.Color,'MarkerSize',25);

nexttile;
plot(t,dV_val)
