%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%         5. practice         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Nonlinear dynamics systems  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%         2019.11.13.         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%       Lorenz system         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Parameters:

% Chaos
sigma = 10;
rho = 28;
beta = 8/3;

start1 = [5;5;5];             % 1. start point
start2 = [4.9;5;5];           % 2. start point
tmax = 50;                    % simulation time
tlim = [0 tmax];              % borders for plotting
xl = [-20,20];
yl = [-30,30];
zl = [-20,50];

Lorenz_equ = @(t,x) [
    sigma*(x(2)-x(1))
    rho*x(1)-x(2)-x(1)*x(3);...
    x(1)*x(2)-beta*x(3)
    ];

% Lorenz equations

figure;
    [ t, x] = ode45(Lorenz_equ, tlim, start1);           % ode45 for the 1st initial position    
    [t2,x2] = ode45(Lorenz_equ, tlim, start2);           % ode45 for the 2nd initial position 
    

% plot x,y,z in the function of time (subplots)
% t-x
subplot(6,2,1);
plot(t,x(:,1), t2,x2(:,1));
xlabel('t');
ylabel('x');
xlim(tlim);
ylim(xl);
grid on; box on;

% t-y
subplot(6,2,3);
plot(t,x(:,2), t2,x2(:,2));
xlabel('t');
ylabel('y');
xlim(tlim);
ylim(yl);
grid on; box on;

% t-z
subplot(6,2,5);
plot(t,x(:,3), t2,x2(:,3));
xlabel('t');
ylabel('z');
xlim(tlim);
ylim(zl);
grid on; box on;

% x-y-z 3D plot
subplot(6,2,7:12);
plot3(x(:,1),x(:,2),x(:,3), x2(:,1),x2(:,2),x2(:,3));
xlabel('x');
ylabel('y');
zlabel('z');
xlim(xl);
ylim( yl);
zlim(zl);
grid on; box on;

subplot(6,2, [2,4,6]);
[Peaks,Location]=findpeaks(x(:,3)); %build in function from the signal processing toolbox: finds local maximums in a vector
peak_n1=Peaks(1:end-1); %plot the 2 datasets in the function of each other
peak_n2=Peaks(2:end);

[Peaks2, my_where2]=findpeaks(x2(:,3)); %build in function from the signal processing toolbox: finds local maximums in a vector
peak2_n1=Peaks2(1:end-1); %plot the 2 datasets in the function of each other
peak2_n2=Peaks2(2:end);

[X,Y] = plotXY(Peaks2);
plot(X,Y,'r'); hold on;

plot(peak_n1, peak_n2, 'b.', peak2_n1, peak2_n2, 'r.'); %plot local maximums: every n+1. in the function the n.
xlim([25 50]);
ylim([25 50]);
xlabel('Zn');
ylabel('Zn+1');
title('Lorenz peak map');
grid on; box on;
hold on
plot([0,46],[0,46])

function [X,Y] = plotXY(x)
    x = x(:)';
    f = x(2:end);
    x = x(1:end-1);
    N = numel(x);
    X = reshape([1;1]*x, [2*N 1]);
    Y = reshape([ 0 f(1:end-1) ; f ],[2*N 1]);
end







