%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%         5. practice         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Nonlinear dynamics systems  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%         2019.11.13.         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%       Lorenz system         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Parameters:

% Chaos
sigma = 10;
rho = 28;
beta = 8/3;

start1 = [5;5;5];             % 1. start point
tmax = 100000;                    % simulation time
tlim = [0 tmax];              % borders for plotting
xl = [-20,20];
yl = [-30,30];
zl = [-20,50];

Lorenz_equ = @(t,x) [
    sigma*(x(2)-x(1))
    rho*x(1)-x(2)-x(1)*x(3);...
    x(1)*x(2)-beta*x(3)
    ];

% Lorenz equations

[t,x] = ode89(Lorenz_equ, tlim, start1);
    
%%
fig = figure(312);
Tl = tiledlayout(1,1,"TileSpacing","compact","Padding","tight","TileIndexing","columnmajor");

ax = nexttile;
[Peaks,Location] = findpeaks(x(:,3)); %build in function from the signal processing toolbox: finds local maximums in a vector
peak_n1 = Peaks(1:end-1); %plot the 2 datasets in the function of each other
peak_n2 = Peaks(2:end);

[X,Y] = plotXY(Peaks);
plot(X,Y,'r'); hold on;

plot(peak_n1,peak_n2,'.b'); %plot local maximums: every n+1. in the function the n.
xlim([25 50]);
ylim([25 50]);
xlabel('Peak');
ylabel('Next peak');
title('Lorenz peak map');
grid on; box on;
hold on
plot([0,46],[0,46])

ax2 = nexttile('south');
hold on; grid on; box on;
plot(t,x(:,3));
plot(t(Location),Peaks,'|r');

fig = figure(313);
Tl = tiledlayout(1,1,"TileSpacing","compact","Padding","tight","TileIndexing","columnmajor");
ax = nexttile;
plot3(x(:,1),x(:,2),x(:,3))


function [X,Y] = plotXY(x)
    x = x(:)';
    f = x(2:end);
    x = x(1:end-1);
    N = numel(x);
    X = reshape([1;1]*x, [2*N 1]);
    Y = reshape([ 0 f(1:end-1) ; f ],[2*N 1]);
end







